package com.readrops.api.services.fever.adapters

import android.annotation.SuppressLint
import com.readrops.api.utils.exceptions.ParseException
import com.readrops.api.utils.extensions.nextNonEmptyString
import com.readrops.api.utils.extensions.nextNullableString
import com.readrops.api.utils.extensions.skipField
import com.readrops.api.utils.extensions.skipToEnd
import com.readrops.db.entities.Feed
import com.squareup.moshi.*

data class FeverFeeds(
        val feeds: List<Feed>,
        val feedsGroups: Map<Int, List<Int>>
)

class FeverFeedsAdapter : JsonAdapter<FeverFeeds>() {

    override fun toJson(writer: JsonWriter, value: FeverFeeds?) {
        // not useful here
    }

    @SuppressLint("CheckResult")
    override fun fromJson(reader: JsonReader): FeverFeeds = with(reader) {
        return try {
            val feeds = arrayListOf<Feed>()
            val feedsGroups = mutableMapOf<Int, List<Int>>()

            beginObject()

            // skip based fields (api_version, auth, last_refreshed...)
            repeat(3) {
                skipField()
            }

            nextName() // beginning of feeds array
            beginArray()

            while (hasNext()) {
                beginObject()

                val feed = Feed()
                while (hasNext()) {
                    with(feed) {
                        when (selectName(NAMES)) {
                            0 -> remoteId = nextInt().toString()
                            1 -> name = nextNonEmptyString()
                            2 -> url = nextNonEmptyString()
                            3 -> siteUrl = nextNullableString()
                            else -> skipValue()
                        }
                    }
                }

                feeds += feed
                endObject()
            }

            endArray()

            nextName()
            beginArray()

            while (hasNext()) {
                beginObject()

                var folderId: Int? = null
                val feedsIds = mutableListOf<Int>()
                while (hasNext()) {
                    when (selectName(JsonReader.Options.of("group_id", "feed_ids"))) {
                        0 -> folderId = nextInt()
                        1 -> feedsIds += nextNonEmptyString().split(",").map { it.toInt() }
                        else -> skipValue()
                    }
                }

                folderId?.let { feedsGroups[it] = feedsIds }
                endObject()
            }

            endArray()
            endObject()

            FeverFeeds(feeds, feedsGroups)
        } catch (e: Exception) {
            throw ParseException(e.message)
        }
    }

    companion object {
        val NAMES: JsonReader.Options = JsonReader.Options.of("id", "title", "url", "site_url")
    }
}